@@ -37,6 +37,8 @@
 #' UPPER CASE, or to Title Case.
 #'
 #' @details
+#' Vectorized over \code{str}.
+#'
 #' \pkg{ICU} implements full Unicode string case mappings. In general,
 #' \itemize{
 #' \item case mapping can change the number of code points and/or code units
@@ -46,6 +48,14 @@
 #'    differently depending on surrounding characters).
 #' }
 #'
+#' With \code{stri_trans_totitle}, if \code{word} \code{BreakIterator}
+#' is used (the default), then the first letter of each word will be capitalized
+#' and the rest will be transformed to lower case.
+#' With a break iterator of type \code{sentence}, the first letter
+#' of each sentence will be capitalized only.
+#' Note that according the \pkg{ICU} User Guide,
+#' the string \code{"one. two. three."} consists of one sentence.
+#'
 #' For more general (but not locale dependent)
 #' text transforms refer to \code{\link{stri_trans_general}}.
 #'
@@ -53,6 +63,11 @@
 #' @param locale \code{NULL} or \code{""} for case mapping following
 #' the conventions of the default locale, or a single string with
 #' locale identifier, see \link{stringi-locale}.
+#' @param opts_brkiter a named list with \pkg{ICU} BreakIterator's settings
+#' as generated with \code{\link{stri_opts_brkiter}};
+#' \code{NULL} for default break iterator, i.e. \code{word};
+#' \code{stri_trans_totitle} only
+#'
 #' @return
 #' Each function returns a character vector.
 #'
@@ -64,26 +79,20 @@
 #' @family transform
 #' @export
 #' @rdname stri_trans_casemap
+#' @family text_boundaries
 #'
 #' @examples
-#'
-#' \dontshow{if (stri_install_check(silent=TRUE))}
+#' \donttest{
 #' stri_trans_toupper("\u00DF", "de_DE") # small German Eszett / scharfes S
-#'
-#' \dontshow{if (stri_install_check(silent=TRUE))}
 #' stri_cmp_eq(stri_trans_toupper("i", "en_US"), stri_trans_toupper("i", "tr_TR"))
-#'
-#' \dontshow{if (stri_install_check(silent=TRUE))}
-#' stri_trans_toupper(c('abc','123','\u0105\u0104'))
-#'
-#' \dontshow{if (stri_install_check(silent=TRUE))}
-#' stri_trans_tolower(c('AbC','123','\u0105\u0104'))
-#'
-#' \dontshow{if (stri_install_check(silent=TRUE))}
-#' stri_trans_totitle(c('AbC','123','\u0105\u0104'))
-#'
-#' \dontshow{if (stri_install_check(silent=TRUE))}
-#' stri_trans_totitle("aLA mA kOTA")
+#' stri_trans_toupper(c('abc', '123', '\u0105\u0104'))
+#' stri_trans_tolower(c('AbC', '123', '\u0105\u0104'))
+#' stri_trans_totitle(c('AbC', '123', '\u0105\u0104'))
+#' stri_trans_totitle("GOOD-OLD cOOkiE mOnSTeR IS watCHinG You. Here HE comes!",
+#'     stri_opts_brkiter(type="word")) # default boundary
+#' stri_trans_totitle("GOOD-OLD cOOkiE mOnSTeR IS watCHinG You. Here HE comes!",
+#'     stri_opts_brkiter(type="sentence"))
+#' }
 stri_trans_tolower <- function(str, locale=NULL) {
    .Call("stri_trans_casemap", str, 1L, locale, PACKAGE="stringi")
 }
@@ -98,6 +107,6 @@ stri_trans_toupper <- function(str, locale=NULL) {
 
 #' @export
 #' @rdname stri_trans_casemap
-stri_trans_totitle <- function(str, locale=NULL) {
-   .Call("stri_trans_casemap", str, 3L, locale, PACKAGE="stringi")
+stri_trans_totitle <- function(str, opts_brkiter=NULL) {
+   .Call("stri_trans_casemap", str, 3L, opts_brkiter, PACKAGE="stringi")
 }
