@@ -39,7 +39,7 @@ using namespace std;
 
 
 /**
- * Extract first or last occurences of a character class in each string
+ * Extract first or last occurrences of a character class in each string
  *
  * @param str character vector
  * @param pattern character vector
@@ -58,14 +58,17 @@ using namespace std;
  *
  * @version 0.2-1 (Marek Gagolewski, 2014-04-05)
  *          StriContainerCharClass now relies on UnicodeSet
+ *
+ * @version 0.3-1 (Marek Gagolewski, 2014-11-04)
+ *    Issue #112: str_prepare_arg* retvals were not PROTECTed from gc
  */
 SEXP stri__extract_firstlast_charclass(SEXP str, SEXP pattern, bool first)
 {
-   str = stri_prepare_arg_string(str, "str");
-   pattern = stri_prepare_arg_string(pattern, "pattern");
+   PROTECT(str = stri_prepare_arg_string(str, "str"));
+   PROTECT(pattern = stri_prepare_arg_string(pattern, "pattern"));
    R_len_t vectorize_length = stri__recycling_rule(true, 2, LENGTH(str), LENGTH(pattern));
 
-   STRI__ERROR_HANDLER_BEGIN
+   STRI__ERROR_HANDLER_BEGIN(2)
    StriContainerUTF8 str_cont(str, vectorize_length);
    StriContainerCharClass pattern_cont(pattern, vectorize_length);
 
@@ -122,7 +125,7 @@ SEXP stri__extract_firstlast_charclass(SEXP str, SEXP pattern, bool first)
 
 
 /**
- * Extract first occurence of a character class in each string
+ * Extract first occurrence of a character class in each string
  *
  * @param str character vector
  * @param pattern character vector
@@ -137,7 +140,7 @@ SEXP stri_extract_first_charclass(SEXP str, SEXP pattern)
 
 
 /**
- * Extract last occurence of a character class in each string
+ * Extract last occurrence of a character class in each string
  *
  * @param str character vector
  * @param pattern character vector
@@ -152,11 +155,13 @@ SEXP stri_extract_last_charclass(SEXP str, SEXP pattern)
 
 
 /**
- * Extract all occurences of a character class in each string
+ * Extract all occurrences of a character class in each string
  *
  * @param str character vector
  * @param pattern character vector
- * @return list of character vectors
+ * @param simplify single logical value
+ *
+ * @return list of character vectors  or character matrix
  *
  * @version 0.1-?? (Marek Gagolewski, 2013-06-08)
  *
@@ -171,18 +176,29 @@ SEXP stri_extract_last_charclass(SEXP str, SEXP pattern)
  *
  * @version 0.2-1 (Marek Gagolewski, 2014-04-05)
  *          StriContainerCharClass now relies on UnicodeSet
+ *
+ * @version 0.3-1 (Marek Gagolewski, 2014-10-24)
+ *          added simplify param
+ *
+ * @version 0.3-1 (Marek Gagolewski, 2014-11-02)
+ *          using StriContainerCharClass::locateAll;
+ *          no longer vectorized over merge
+ *
+ * @version 0.3-1 (Marek Gagolewski, 2014-11-04)
+ *    Issue #112: str_prepare_arg* retvals were not PROTECTed from gc
  */
-SEXP stri_extract_all_charclass(SEXP str, SEXP pattern, SEXP merge)
+SEXP stri_extract_all_charclass(SEXP str, SEXP pattern, SEXP merge, SEXP simplify)
 {
-   str = stri_prepare_arg_string(str, "str");
-   pattern = stri_prepare_arg_string(pattern, "pattern");
-   merge = stri_prepare_arg_logical(merge, "merge");
-   R_len_t vectorize_length = stri__recycling_rule(true, 3, LENGTH(str), LENGTH(pattern), LENGTH(merge));
+   PROTECT(str = stri_prepare_arg_string(str, "str"));
+   PROTECT(pattern = stri_prepare_arg_string(pattern, "pattern"));
+   bool merge_cur = stri__prepare_arg_logical_1_notNA(merge, "merge");
+   bool simplify1 = stri__prepare_arg_logical_1_notNA(simplify, "simplify");
+   R_len_t vectorize_length = stri__recycling_rule(true, 2,
+      LENGTH(str), LENGTH(pattern));
 
-   STRI__ERROR_HANDLER_BEGIN
+   STRI__ERROR_HANDLER_BEGIN(2)
    StriContainerUTF8 str_cont(str, vectorize_length);
    StriContainerCharClass pattern_cont(pattern, vectorize_length);
-   StriContainerLogical merge_cont(merge, vectorize_length);
 
    SEXP notfound; // this vector will be set iff not found or NA
    STRI__PROTECT(notfound = stri__vector_NA_strings(1));
@@ -194,73 +210,41 @@ SEXP stri_extract_all_charclass(SEXP str, SEXP pattern, SEXP merge)
          i != pattern_cont.vectorize_end();
          i = pattern_cont.vectorize_next(i))
    {
-      if (pattern_cont.isNA(i) || str_cont.isNA(i) || merge_cont.isNA(i)) {
+      if (pattern_cont.isNA(i) || str_cont.isNA(i)) {
          SET_VECTOR_ELT(ret, i, notfound);
          continue;
       }
 
-      bool merge_cur = merge_cont.get(i);
-      const UnicodeSet* pattern_cur = &pattern_cont.get(i);
-      R_len_t     str_cur_n = str_cont.get(i).length();
+      R_len_t str_cur_n     = str_cont.get(i).length();
       const char* str_cur_s = str_cont.get(i).c_str();
-      R_len_t j, jlast;
-      UChar32 chr;
-      deque< pair<R_len_t, R_len_t> > occurences; // codepoint based-indices
+      deque< pair<R_len_t, R_len_t> > occurrences;
+      StriContainerCharClass::locateAll(
+         occurrences, &pattern_cont.get(i),
+         str_cur_s, str_cur_n, merge_cur,
+         false /* byte-based indices */
+      );
 
-      for (jlast=j=0; j<str_cur_n; ) {
-         U8_NEXT(str_cur_s, j, str_cur_n, chr);
-         if (chr < 0) // invalid utf-8 sequence
-            throw StriException(MSG__INVALID_UTF8);
-         if (pattern_cur->contains(chr)) {
-            occurences.push_back(pair<R_len_t, R_len_t>(jlast, j));
-         }
-         jlast = j;
-      }
-
-      R_len_t noccurences = (R_len_t)occurences.size();
-      if (noccurences == 0)
+      R_len_t noccurrences = (R_len_t)occurrences.size();
+      if (noccurrences == 0) {
          SET_VECTOR_ELT(ret, i, notfound);
-      else if (merge_cur && noccurences > 1) {
-         // do merge
-         deque< pair<R_len_t, R_len_t> > occurences2;
-         deque< pair<R_len_t, R_len_t> >::iterator iter = occurences.begin();
-         occurences2.push_back(*iter);
-         for (++iter; iter != occurences.end(); ++iter) {
-            pair<R_len_t, R_len_t> curoccur = *iter;
-            if (occurences2.back().second == curoccur.first) { // continue seq
-               occurences2.back().second = curoccur.second;  // change `end`
-            }
-            else { // new seq
-               occurences2.push_back(curoccur);
-            }
-         }
+         continue;
+      }
 
-         // create resulting matrix from occurences2
-         R_len_t noccurences2 = (R_len_t)occurences2.size();
-         SEXP cur_res;
-         STRI__PROTECT(cur_res = Rf_allocVector(STRSXP, noccurences2));
-         iter = occurences2.begin();
-         for (R_len_t f = 0; iter != occurences2.end(); ++iter, ++f) {
-            pair<R_len_t, R_len_t> curo = *iter;
-            SET_STRING_ELT(cur_res, f,
-               Rf_mkCharLenCE(str_cur_s+curo.first, curo.second-curo.first, CE_UTF8));
-         }
-         SET_VECTOR_ELT(ret, i, cur_res);
-         STRI__UNPROTECT(1)
-      }
-      else {
-         // do not merge
-         SEXP cur_res;
-         STRI__PROTECT(cur_res = Rf_allocVector(STRSXP, noccurences));
-         deque< pair<R_len_t, R_len_t> >::iterator iter = occurences.begin();
-         for (R_len_t f = 0; iter != occurences.end(); ++iter, ++f) {
-            pair<R_len_t, R_len_t> curo = *iter;
-            SET_STRING_ELT(cur_res, f,
-               Rf_mkCharLenCE(str_cur_s+curo.first, curo.second-curo.first, CE_UTF8));
-         }
-         SET_VECTOR_ELT(ret, i, cur_res);
-         STRI__UNPROTECT(1)
+      SEXP cur_res;
+      STRI__PROTECT(cur_res = Rf_allocVector(STRSXP, noccurrences));
+      deque< pair<R_len_t, R_len_t> >::iterator iter = occurrences.begin();
+      for (R_len_t f = 0; iter != occurrences.end(); ++iter, ++f) {
+         pair<R_len_t, R_len_t> curo = *iter;
+         SET_STRING_ELT(cur_res, f,
+            Rf_mkCharLenCE(str_cur_s+curo.first, curo.second-curo.first, CE_UTF8));
       }
+      SET_VECTOR_ELT(ret, i, cur_res);
+      STRI__UNPROTECT(1)
+   }
+
+   if (simplify1) {
+      ret = stri_list2matrix(ret, Rf_ScalarLogical(TRUE),
+         stri__vector_NA_strings(1));
    }
 
    STRI__UNPROTECT_ALL
