@@ -33,7 +33,6 @@
 #include "stri_stringi.h"
 #include "stri_container_utf8.h"
 #include "stri_container_bytesearch.h"
-#include <unicode/uregex.h>
 
 
 /**
@@ -56,13 +55,16 @@
  *
  * @version 0.2-3 (Marek Gagolewski, 2014-05-08)
  *          stri_detect_fixed now uses byte search only
+ *
+ * @version 0.3-1 (Marek Gagolewski, 2014-11-05)
+ *    Issue #112: str_prepare_arg* retvals were not PROTECTed from gc
  */
 SEXP stri_detect_fixed(SEXP str, SEXP pattern)
 {
-   str = stri_prepare_arg_string(str, "str");
-   pattern = stri_prepare_arg_string(pattern, "pattern");
+   PROTECT(str = stri_prepare_arg_string(str, "str"));
+   PROTECT(pattern = stri_prepare_arg_string(pattern, "pattern"));
 
-   STRI__ERROR_HANDLER_BEGIN
+   STRI__ERROR_HANDLER_BEGIN(2)
    int vectorize_length = stri__recycling_rule(true, 2, LENGTH(str), LENGTH(pattern));
    StriContainerUTF8 str_cont(str, vectorize_length);
    StriContainerByteSearch pattern_cont(pattern, vectorize_length);
