@@ -50,7 +50,10 @@ using namespace std;
  * @param pattern character vector
  * @param n_max integer vector
  * @param omit_empty logical vector
+ * @param tokens_only single logical value
+ * @param simplify single logical value
  *
+ * @return list of character vectors  or character matrix
  *
  * @version 0.1-?? (Bartek Tartanus)
  *
@@ -62,20 +65,36 @@ using namespace std;
  *
  * @version 0.2-3 (Marek Gagolewski, 2014-05-08)
  *          stri_split_fixed now uses byte search only
+ *
+ * @version 0.3-1 (Marek Gagolewski, 2014-10-19)
+ *          added tokens_only param
+ *
+ * @version 0.3-1 (Marek Gagolewski, 2014-10-23)
+ *          added split param
+ *
+ * @version 0.3-1 (Marek Gagolewski, 2014-10-24)
+ *          allow omit_empty=NA
+ *
+ * @version 0.3-1 (Marek Gagolewski, 2014-11-05)
+ *    Issue #112: str_prepare_arg* retvals were not PROTECTed from gc
  */
-SEXP stri_split_fixed(SEXP str, SEXP pattern, SEXP n_max, SEXP omit_empty)
+SEXP stri_split_fixed(SEXP str, SEXP pattern, SEXP n_max,
+                      SEXP omit_empty, SEXP tokens_only, SEXP simplify)
 {
-   str = stri_prepare_arg_string(str, "str");
-   pattern = stri_prepare_arg_string(pattern, "pattern");
-   n_max = stri_prepare_arg_integer(n_max, "n_max");
-   omit_empty = stri_prepare_arg_logical(omit_empty, "omit_empty");
+   PROTECT(str = stri_prepare_arg_string(str, "str"));
+   PROTECT(pattern = stri_prepare_arg_string(pattern, "pattern"));
+   PROTECT(n_max = stri_prepare_arg_integer(n_max, "n_max"));
+   PROTECT(omit_empty = stri_prepare_arg_logical(omit_empty, "omit_empty"));
+   bool tokens_only1 = stri__prepare_arg_logical_1_notNA(tokens_only, "tokens_only");
+   bool simplify1 = stri__prepare_arg_logical_1_notNA(simplify, "simplify");
 
-   STRI__ERROR_HANDLER_BEGIN
-   R_len_t vectorize_length = stri__recycling_rule(true, 4, LENGTH(str), LENGTH(pattern), LENGTH(n_max), LENGTH(omit_empty));
+   STRI__ERROR_HANDLER_BEGIN(4)
+   R_len_t vectorize_length = stri__recycling_rule(true, 4,
+      LENGTH(str), LENGTH(pattern), LENGTH(n_max), LENGTH(omit_empty));
    StriContainerUTF8 str_cont(str, vectorize_length);
    StriContainerByteSearch pattern_cont(pattern, vectorize_length);
-   StriContainerInteger   n_max_cont(n_max, vectorize_length);
-   StriContainerLogical   omit_empty_cont(omit_empty, vectorize_length);
+   StriContainerInteger n_max_cont(n_max, vectorize_length);
+   StriContainerLogical omit_empty_cont(omit_empty, vectorize_length);
 
    SEXP ret;
    STRI__PROTECT(ret = Rf_allocVector(VECSXP, vectorize_length));
@@ -84,26 +103,32 @@ SEXP stri_split_fixed(SEXP str, SEXP pattern, SEXP n_max, SEXP omit_empty)
          i != pattern_cont.vectorize_end();
          i = pattern_cont.vectorize_next(i))
    {
-      if (n_max_cont.isNA(i) || omit_empty_cont.isNA(i)) {
+      if (n_max_cont.isNA(i)) {
          SET_VECTOR_ELT(ret, i, stri__vector_NA_strings(1));
          continue;
       }
       int  n_max_cur        = n_max_cont.get(i);
-      int  omit_empty_cur   = omit_empty_cont.get(i);
+      int  omit_empty_cur   = !omit_empty_cont.isNA(i) && omit_empty_cont.get(i);
 
       STRI__CONTINUE_ON_EMPTY_OR_NA_STR_PATTERN(str_cont, pattern_cont,
          SET_VECTOR_ELT(ret, i, stri__vector_NA_strings(1));,
-         SET_VECTOR_ELT(ret, i, stri__vector_empty_strings((omit_empty_cur || n_max_cur == 0)?0:1));)
+         SET_VECTOR_ELT(ret, i,
+            (omit_empty_cont.isNA(i))?stri__vector_NA_strings(1):
+            stri__vector_empty_strings((omit_empty_cur || n_max_cur == 0)?0:1));)
 
       R_len_t     str_cur_n = str_cont.get(i).length();
       const char* str_cur_s = str_cont.get(i).c_str();
 
-      if (n_max_cur < 0)
+      if (n_max_cur >= INT_MAX-1)
+         throw StriException(MSG__EXPECTED_SMALLER, "n_max");
+      else if (n_max_cur < 0)
          n_max_cur = INT_MAX;
       else if (n_max_cur == 0) {
          SET_VECTOR_ELT(ret, i, Rf_allocVector(STRSXP, 0));
          continue;
       }
+      else if (tokens_only1)
+         n_max_cur++; // we need to do one split ahead here
 
       pattern_cont.setupMatcherFwd(i, str_cur_s, str_cur_n);
       R_len_t k;
@@ -115,10 +140,10 @@ SEXP stri_split_fixed(SEXP str, SEXP pattern, SEXP n_max, SEXP omit_empty)
          R_len_t s2 = (R_len_t)pattern_cont.getMatchedLength() + s1;
 
          if (omit_empty_cur && fields.back().first == s1)
-            fields.back().first = s2; // don't start new field
+            fields.back().first = s2; // don't start any new field
          else {
             fields.back().second = s1;
-            fields.push_back(pair<R_len_t, R_len_t>(s2, s2)); // start new field here
+            fields.push_back(pair<R_len_t, R_len_t>(s2, s2)); // start a new field here
             ++k; // another field
          }
       }
@@ -126,20 +151,35 @@ SEXP stri_split_fixed(SEXP str, SEXP pattern, SEXP n_max, SEXP omit_empty)
       if (omit_empty_cur && fields.back().first == fields.back().second)
          fields.pop_back();
 
+      if (tokens_only1 && n_max_cur < INT_MAX) {
+         n_max_cur--; // one split ahead could have been made, see above
+         while (fields.size() > (size_t)n_max_cur)
+            fields.pop_back(); // get rid of the remainder
+      }
+
       SEXP ans;
       STRI__PROTECT(ans = Rf_allocVector(STRSXP, fields.size()));
 
       deque< pair<R_len_t, R_len_t> >::iterator iter = fields.begin();
       for (k = 0; iter != fields.end(); ++iter, ++k) {
          pair<R_len_t, R_len_t> curoccur = *iter;
-         SET_STRING_ELT(ans, k,
-            Rf_mkCharLenCE(str_cur_s+curoccur.first, curoccur.second-curoccur.first, CE_UTF8));
+         if (curoccur.second == curoccur.first && omit_empty_cont.isNA(i))
+            SET_STRING_ELT(ans, k, NA_STRING);
+         else
+            SET_STRING_ELT(ans, k,
+               Rf_mkCharLenCE(str_cur_s+curoccur.first,
+                  curoccur.second-curoccur.first, CE_UTF8));
       }
 
       SET_VECTOR_ELT(ret, i, ans);
       STRI__UNPROTECT(1);
    }
 
+   if (simplify1) {
+      ret = stri_list2matrix(ret, Rf_ScalarLogical(TRUE),
+         stri__vector_NA_strings(1));
+   }
+
    STRI__UNPROTECT_ALL
    return ret;
    STRI__ERROR_HANDLER_END(; /* nothing interesting on error */)
