@@ -52,25 +52,30 @@ using namespace std;
  * @return character vector
  *
  * @version 0.1-?? (Marek Gagolewski, 2013-08-04)
+ *
+ * @version 0.3-1 (Marek Gagolewski, 2014-11-05)
+ *    Issue #112: str_prepare_arg* retvals were not PROTECTed from gc
  */
 SEXP stri_split_lines1(SEXP str)
 {
-   str = stri_prepare_arg_string_1(str, "str");
+   PROTECT(str = stri_prepare_arg_string_1(str, "str"));
    R_len_t vectorize_length = LENGTH(str);
 
-   STRI__ERROR_HANDLER_BEGIN
+   STRI__ERROR_HANDLER_BEGIN(1)
    StriContainerUTF8 str_cont(str, vectorize_length);
 
-   if (str_cont.isNA(0))
+   if (str_cont.isNA(0)) {
+      STRI__UNPROTECT_ALL
       return str;
+   }
 
    const char* str_cur_s = str_cont.get(0).c_str();
    R_len_t str_cur_n = str_cont.get(0).length();
 
    UChar32 c;
    R_len_t jlast;
-   deque< pair<R_len_t, R_len_t> > occurences;
-   occurences.push_back(pair<R_len_t, R_len_t>(0, 0));
+   deque< pair<R_len_t, R_len_t> > occurrences;
+   occurrences.push_back(pair<R_len_t, R_len_t>(0, 0));
    for (R_len_t j=0; j < str_cur_n; /* null */) {
       jlast = j;
       U8_NEXT(str_cur_s, j, str_cur_n, c);
@@ -103,19 +108,19 @@ SEXP stri_split_lines1(SEXP str)
 
          default:
             /* not a newline character */
-            occurences.back().second = j;
+            occurrences.back().second = j;
             continue;
       }
 
-      occurences.back().second = jlast;
+      occurrences.back().second = jlast;
       if (j < str_cur_n)
-         occurences.push_back(pair<R_len_t, R_len_t>(j, j));
+         occurrences.push_back(pair<R_len_t, R_len_t>(j, j));
    }
 
    SEXP ans;
-   STRI__PROTECT(ans = Rf_allocVector(STRSXP, (R_len_t)occurences.size()));
-   deque< pair<R_len_t, R_len_t> >::iterator iter = occurences.begin();
-   for (R_len_t k = 0; iter != occurences.end(); ++iter, ++k) {
+   STRI__PROTECT(ans = Rf_allocVector(STRSXP, (R_len_t)occurrences.size()));
+   deque< pair<R_len_t, R_len_t> >::iterator iter = occurrences.begin();
+   for (R_len_t k = 0; iter != occurrences.end(); ++iter, ++k) {
       pair<R_len_t, R_len_t> curoccur = *iter;
       SET_STRING_ELT(ans, k,
          Rf_mkCharLenCE(str_cur_s+curoccur.first, curoccur.second-curoccur.first, CE_UTF8));
@@ -131,23 +136,28 @@ SEXP stri_split_lines1(SEXP str)
  * Split a string into text lines
  *
  * @param str character vector
- * @param n_max integer vector
  * @param omit_empty logical vector
  *
  * @return list of character vectors
  *
  * @version 0.1-?? (Marek Gagolewski, 2013-08-04)
+ *
+ * @version 0.3-1 (Marek Gagolewski, 2014-10-30)
+ *                removed `n_max` arg, as it doesn't make sense
+ *
+ * @version 0.3-1 (Marek Gagolewski, 2014-11-05)
+ *    Issue #112: str_prepare_arg* retvals were not PROTECTed from gc
  */
-SEXP stri_split_lines(SEXP str, SEXP n_max, SEXP omit_empty)
+SEXP stri_split_lines(SEXP str, SEXP omit_empty)
 {
-   str = stri_prepare_arg_string(str, "str");
-   n_max = stri_prepare_arg_integer(n_max, "n_max");
-   omit_empty = stri_prepare_arg_logical(omit_empty, "omit_empty");
-   R_len_t vectorize_length = stri__recycling_rule(true, 3, LENGTH(str), LENGTH(n_max), LENGTH(omit_empty));
+   PROTECT(str = stri_prepare_arg_string(str, "str"));
+//   n_max = stri_prepare_arg_integer(n_max, "n_max");
+   PROTECT(omit_empty = stri_prepare_arg_logical(omit_empty, "omit_empty"));
+   R_len_t vectorize_length = stri__recycling_rule(true, 2, LENGTH(str), /*LENGTH(n_max), */LENGTH(omit_empty));
 
-   STRI__ERROR_HANDLER_BEGIN
+   STRI__ERROR_HANDLER_BEGIN(2)
    StriContainerUTF8 str_cont(str, vectorize_length);
-   StriContainerInteger   n_max_cont(n_max, vectorize_length);
+//   StriContainerInteger   n_max_cont(n_max, vectorize_length);
    StriContainerLogical   omit_empty_cont(omit_empty, vectorize_length);
 
    SEXP ret;
@@ -164,15 +174,15 @@ SEXP stri_split_lines(SEXP str, SEXP n_max, SEXP omit_empty)
 
       const char* str_cur_s = str_cont.get(i).c_str();
       R_len_t str_cur_n = str_cont.get(i).length();
-      int  n_max_cur        = n_max_cont.get(i);
+//      int  n_max_cur        = n_max_cont.get(i);
       int  omit_empty_cur   = omit_empty_cont.get(i);
 
-      if (n_max_cur < 0)
-         n_max_cur = INT_MAX;
-      else if (n_max_cur == 0) {
-         SET_VECTOR_ELT(ret, i, Rf_allocVector(STRSXP, 0));
-         continue;
-      }
+//      if (n_max_cur < 0)
+//         n_max_cur = INT_MAX;
+//      else if (n_max_cur == 0) {
+//         SET_VECTOR_ELT(ret, i, Rf_allocVector(STRSXP, 0));
+//         continue;
+//      }
 
 //#define STRI_INDEX_NEWLINE_CR   0
 //#define STRI_INDEX_NEWLINE_LF   1
@@ -190,9 +200,9 @@ SEXP stri_split_lines(SEXP str, SEXP n_max, SEXP omit_empty)
 
       UChar32 c;
       R_len_t jlast, k=1;
-      deque< pair<R_len_t, R_len_t> > occurences;
-      occurences.push_back(pair<R_len_t, R_len_t>(0, 0));
-      for (R_len_t j=0; j < str_cur_n && k < n_max_cur; /* null */) {
+      deque< pair<R_len_t, R_len_t> > occurrences;
+      occurrences.push_back(pair<R_len_t, R_len_t>(0, 0));
+      for (R_len_t j=0; j < str_cur_n /*&& k < n_max_cur*/; /* null */) {
          jlast = j;
          U8_NEXT(str_cur_s, j, str_cur_n, c);
 
@@ -233,30 +243,30 @@ SEXP stri_split_lines(SEXP str, SEXP n_max, SEXP omit_empty)
 
             default:
                /* not a newline character */
-               occurences.back().second = j;
+               occurrences.back().second = j;
                continue;
          }
 
          // if here, then at newline
-         if (omit_empty_cur && occurences.back().second == occurences.back().first)
-            occurences.back().first = occurences.back().second = j; // don't start new field
+         if (omit_empty_cur && occurrences.back().second == occurrences.back().first)
+            occurrences.back().first = occurrences.back().second = j; // don't start new field
          else {
-            occurences.back().second = jlast;
-            occurences.push_back(pair<R_len_t, R_len_t>(j, j));
+            occurrences.back().second = jlast;
+            occurrences.push_back(pair<R_len_t, R_len_t>(j, j));
             ++k; // another field
          }
       }
 
-      if (k == n_max_cur)
-         occurences.back().second = str_cur_n;
-      if (omit_empty_cur && occurences.back().first == occurences.back().second)
-         occurences.pop_back();
+//      if (k == n_max_cur)
+//         occurrences.back().second = str_cur_n;
+      if (omit_empty_cur && occurrences.back().first == occurrences.back().second)
+         occurrences.pop_back();
 
       SEXP ans;
-      STRI__PROTECT(ans = Rf_allocVector(STRSXP, (R_len_t)occurences.size()));
+      STRI__PROTECT(ans = Rf_allocVector(STRSXP, (R_len_t)occurrences.size()));
 
-      deque< pair<R_len_t, R_len_t> >::iterator iter = occurences.begin();
-      for (R_len_t l = 0; iter != occurences.end(); ++iter, ++l) {
+      deque< pair<R_len_t, R_len_t> >::iterator iter = occurrences.begin();
+      for (R_len_t l = 0; iter != occurrences.end(); ++iter, ++l) {
          pair<R_len_t, R_len_t> curoccur = *iter;
          SET_STRING_ELT(ans, l,
             Rf_mkCharLenCE(str_cur_s+curoccur.first, curoccur.second-curoccur.first, CE_UTF8));
