@@ -39,26 +39,29 @@ using namespace std;
 
 
 /**
- * Extract first occurence of a regex pattern in each string
+ * Extract first occurrence of a regex pattern in each string
  *
  * @param str character vector
  * @param pattern character vector
  * @param opts_regex list
- * @param first logical - search for the first or the last occurence?
+ * @param first logical - search for the first or the last occurrence?
  * @return character vector
  *
  * @version 0.1-?? (Marek Gagolewski, 2013-06-20)
+ *
+ * @version 0.3-1 (Marek Gagolewski, 2014-11-05)
+ *    Issue #112: str_prepare_arg* retvals were not PROTECTed from gc
  */
 SEXP stri__extract_firstlast_regex(SEXP str, SEXP pattern, SEXP opts_regex, bool first)
 {
-   str = stri_prepare_arg_string(str, "str"); // prepare string argument
-   pattern = stri_prepare_arg_string(pattern, "pattern"); // prepare string argument
+   PROTECT(str = stri_prepare_arg_string(str, "str")); // prepare string argument
+   PROTECT(pattern = stri_prepare_arg_string(pattern, "pattern")); // prepare string argument
    R_len_t vectorize_length = stri__recycling_rule(true, 2, LENGTH(str), LENGTH(pattern));
 
    uint32_t pattern_flags = StriContainerRegexPattern::getRegexFlags(opts_regex);
 
    UText* str_text = NULL; // may potentially be slower, but definitely is more convenient!
-   STRI__ERROR_HANDLER_BEGIN
+   STRI__ERROR_HANDLER_BEGIN(2)
    StriContainerUTF8 str_cont(str, vectorize_length);
    StriContainerRegexPattern pattern_cont(pattern, vectorize_length, pattern_flags);
 
@@ -112,7 +115,7 @@ SEXP stri__extract_firstlast_regex(SEXP str, SEXP pattern, SEXP opts_regex, bool
 
 
 /**
- * Extract first occurence of a regex pattern in each string
+ * Extract first occurrence of a regex pattern in each string
  *
  * @param str character vector
  * @param pattern character vector
@@ -128,7 +131,7 @@ SEXP stri_extract_first_regex(SEXP str, SEXP pattern, SEXP opts_regex)
 
 
 /**
- * Extract last occurence of a regex pattern in each string
+ * Extract last occurrence of a regex pattern in each string
  *
  * @param str character vector
  * @param pattern character vector
@@ -144,25 +147,33 @@ SEXP stri_extract_last_regex(SEXP str, SEXP pattern, SEXP opts_regex)
 
 
 /**
- * Extract all occurences of a regex pattern in each string
+ * Extract all occurrences of a regex pattern in each string
  *
  * @param str character vector
  * @param pattern character vector
  * @param opts_regex list
- * @return list of character vectors
+ * @param simplify single logical value
+ *
+ * @return list of character vectors  or character matrix
  *
  * @version 0.1-?? (Marek Gagolewski, 2013-06-20)
+ *
+ * @version 0.3-1 (Marek Gagolewski, 2014-10-24)
+ *          added simplify param
+ *
+ * @version 0.3-1 (Marek Gagolewski, 2014-11-05)
+ *    Issue #112: str_prepare_arg* retvals were not PROTECTed from gc
  */
-SEXP stri_extract_all_regex(SEXP str, SEXP pattern, SEXP opts_regex)
+SEXP stri_extract_all_regex(SEXP str, SEXP pattern, SEXP simplify, SEXP opts_regex)
 {
-   str = stri_prepare_arg_string(str, "str"); // prepare string argument
-   pattern = stri_prepare_arg_string(pattern, "pattern"); // prepare string argument
+   PROTECT(str = stri_prepare_arg_string(str, "str")); // prepare string argument
+   PROTECT(pattern = stri_prepare_arg_string(pattern, "pattern")); // prepare string argument
    R_len_t vectorize_length = stri__recycling_rule(true, 2, LENGTH(str), LENGTH(pattern));
-
+   bool simplify1 = stri__prepare_arg_logical_1_notNA(simplify, "simplify");
    uint32_t pattern_flags = StriContainerRegexPattern::getRegexFlags(opts_regex);
 
    UText* str_text = NULL; // may potentially be slower, but definitely is more convenient!
-   STRI__ERROR_HANDLER_BEGIN
+   STRI__ERROR_HANDLER_BEGIN(2)
    StriContainerUTF8 str_cont(str, vectorize_length);
    StriContainerRegexPattern pattern_cont(pattern, vectorize_length, pattern_flags);
 
@@ -184,25 +195,25 @@ SEXP stri_extract_all_regex(SEXP str, SEXP pattern, SEXP opts_regex)
 
       matcher->reset(str_text);
 
-      deque< pair<R_len_t, R_len_t> > occurences;
+      deque< pair<R_len_t, R_len_t> > occurrences;
       while ((int)matcher->find()) {
-         occurences.push_back(pair<R_len_t, R_len_t>(
+         occurrences.push_back(pair<R_len_t, R_len_t>(
             (R_len_t)matcher->start(status), (R_len_t)matcher->end(status)
          ));
          if (U_FAILURE(status)) throw StriException(status);
       }
 
-      R_len_t noccurences = (R_len_t)occurences.size();
-      if (noccurences <= 0) {
+      R_len_t noccurrences = (R_len_t)occurrences.size();
+      if (noccurrences <= 0) {
          SET_VECTOR_ELT(ret, i, stri__vector_NA_strings(1));
          continue;
       }
 
       const char* str_cur_s = str_cont.get(i).c_str();
       SEXP cur_res;
-      STRI__PROTECT(cur_res = Rf_allocVector(STRSXP, noccurences));
-      deque< pair<R_len_t, R_len_t> >::iterator iter = occurences.begin();
-      for (R_len_t j = 0; iter != occurences.end(); ++iter, ++j) {
+      STRI__PROTECT(cur_res = Rf_allocVector(STRSXP, noccurrences));
+      deque< pair<R_len_t, R_len_t> >::iterator iter = occurrences.begin();
+      for (R_len_t j = 0; iter != occurrences.end(); ++iter, ++j) {
          pair<R_len_t, R_len_t> curo = *iter;
          SET_STRING_ELT(cur_res, j,
             Rf_mkCharLenCE(str_cur_s+curo.first, curo.second-curo.first, CE_UTF8));
@@ -215,6 +226,12 @@ SEXP stri_extract_all_regex(SEXP str, SEXP pattern, SEXP opts_regex)
       utext_close(str_text);
       str_text = NULL;
    }
+
+   if (simplify1) {
+      ret = stri_list2matrix(ret, Rf_ScalarLogical(TRUE),
+         stri__vector_NA_strings(1));
+   }
+
    STRI__UNPROTECT_ALL
    return ret;
    STRI__ERROR_HANDLER_END(if (str_text) utext_close(str_text);)
