@@ -50,7 +50,10 @@ using namespace std;
  * @param pattern character vector
  * @param n_max integer vector
  * @param opts_regex
- * @return list of character vectors
+ * @param tokens_only single logical value
+ * @param simplify single logical value
+ *
+ * @return list of character vectors  or character matrix
  *
  * @version 0.1-?? (Marek Gagolewski, 2013-06-21)
  *
@@ -59,20 +62,35 @@ using namespace std;
  *
  * @version 0.1-24 (Marek Gagolewski, 2014-03-11)
  *          Added missing utext_close call to avoid memleaks
+ *
+ * @version 0.3-1 (Marek Gagolewski, 2014-10-19)
+ *          added tokens_only param
+ *
+ * @version 0.3-1 (Marek Gagolewski, 2014-10-23)
+ *          added split param
+ *
+ * @version 0.3-1 (Marek Gagolewski, 2014-10-24)
+ *          allow omit_empty=NA
+ *
+ * @version 0.3-1 (Marek Gagolewski, 2014-11-05)
+ *    Issue #112: str_prepare_arg* retvals were not PROTECTed from gc
  */
-SEXP stri_split_regex(SEXP str, SEXP pattern, SEXP n_max, SEXP omit_empty, SEXP opts_regex)
+SEXP stri_split_regex(SEXP str, SEXP pattern, SEXP n_max, SEXP omit_empty,
+                      SEXP tokens_only, SEXP simplify, SEXP opts_regex)
 {
-   str = stri_prepare_arg_string(str, "str");
-   pattern = stri_prepare_arg_string(pattern, "pattern");
-   n_max = stri_prepare_arg_integer(n_max, "n_max");
-   omit_empty = stri_prepare_arg_logical(omit_empty, "omit_empty");
-   R_len_t vectorize_length = stri__recycling_rule(true, 4, LENGTH(str),
-         LENGTH(pattern), LENGTH(n_max), LENGTH(omit_empty));
+   PROTECT(str = stri_prepare_arg_string(str, "str"));
+   PROTECT(pattern = stri_prepare_arg_string(pattern, "pattern"));
+   PROTECT(n_max = stri_prepare_arg_integer(n_max, "n_max"));
+   PROTECT(omit_empty = stri_prepare_arg_logical(omit_empty, "omit_empty"));
+   bool tokens_only1 = stri__prepare_arg_logical_1_notNA(tokens_only, "tokens_only");
+   bool simplify1 = stri__prepare_arg_logical_1_notNA(simplify, "simplify");
+   R_len_t vectorize_length = stri__recycling_rule(true, 4,
+      LENGTH(str), LENGTH(pattern), LENGTH(n_max), LENGTH(omit_empty));
 
    uint32_t pattern_flags = StriContainerRegexPattern::getRegexFlags(opts_regex);
 
    UText* str_text = NULL; // may potentially be slower, but definitely is more convenient!
-   STRI__ERROR_HANDLER_BEGIN
+   STRI__ERROR_HANDLER_BEGIN(4)
    StriContainerUTF8      str_cont(str, vectorize_length);
    StriContainerInteger   n_max_cont(n_max, vectorize_length);
    StriContainerLogical   omit_empty_cont(omit_empty, vectorize_length);
@@ -85,27 +103,33 @@ SEXP stri_split_regex(SEXP str, SEXP pattern, SEXP n_max, SEXP omit_empty, SEXP
          i != pattern_cont.vectorize_end();
          i = pattern_cont.vectorize_next(i))
    {
-      if (n_max_cont.isNA(i) || omit_empty_cont.isNA(i)) {
+      if (n_max_cont.isNA(i)) {
          SET_VECTOR_ELT(ret, i, stri__vector_NA_strings(1));
          continue;
       }
 
       int  n_max_cur        = n_max_cont.get(i);
-      int  omit_empty_cur   = omit_empty_cont.get(i);
+      int  omit_empty_cur   = !omit_empty_cont.isNA(i) && omit_empty_cont.get(i);
 
       STRI__CONTINUE_ON_EMPTY_OR_NA_STR_PATTERN(str_cont, pattern_cont,
          SET_VECTOR_ELT(ret, i, stri__vector_NA_strings(1));,
-         SET_VECTOR_ELT(ret, i, stri__vector_empty_strings((omit_empty_cur || n_max_cur == 0)?0:1));)
+         SET_VECTOR_ELT(ret, i,
+            (omit_empty_cont.isNA(i))?stri__vector_NA_strings(1):
+            stri__vector_empty_strings((omit_empty_cur || n_max_cur == 0)?0:1));)
 
       R_len_t     str_cur_n = str_cont.get(i).length();
       const char* str_cur_s = str_cont.get(i).c_str();
 
-      if (n_max_cur < 0)
+      if (n_max_cur >= INT_MAX-1)
+         throw StriException(MSG__EXPECTED_SMALLER, "n_max");
+      else if (n_max_cur < 0)
          n_max_cur = INT_MAX;
       else if (n_max_cur == 0) {
          SET_VECTOR_ELT(ret, i, Rf_allocVector(STRSXP, 0));
          continue;
       }
+      else if (tokens_only1)
+         n_max_cur++; // we need to do one split ahead here
 
       UErrorCode status = U_ZERO_ERROR;
       RegexMatcher *matcher = pattern_cont.getMatcher(i); // will be deleted automatically
@@ -136,13 +160,22 @@ SEXP stri_split_regex(SEXP str, SEXP pattern, SEXP n_max, SEXP omit_empty, SEXP
       if (omit_empty_cur && fields.back().first == fields.back().second)
          fields.pop_back();
 
+      if (tokens_only1 && n_max_cur < INT_MAX) {
+         n_max_cur--; // one split ahead could have been made, see above
+         while (fields.size() > (size_t)n_max_cur)
+            fields.pop_back(); // get rid of the remainder
+      }
+
       SEXP ans;
       STRI__PROTECT(ans = Rf_allocVector(STRSXP, fields.size()));
 
       deque< pair<R_len_t, R_len_t> >::iterator iter = fields.begin();
       for (k = 0; iter != fields.end(); ++iter, ++k) {
          pair<R_len_t, R_len_t> curoccur = *iter;
-         SET_STRING_ELT(ans, k,
+         if (curoccur.second == curoccur.first && omit_empty_cont.isNA(i))
+            SET_STRING_ELT(ans, k, NA_STRING);
+         else
+            SET_STRING_ELT(ans, k,
             Rf_mkCharLenCE(str_cur_s+curoccur.first, curoccur.second-curoccur.first, CE_UTF8));
       }
 
@@ -155,6 +188,11 @@ SEXP stri_split_regex(SEXP str, SEXP pattern, SEXP n_max, SEXP omit_empty, SEXP
       str_text = NULL;
    }
 
+   if (simplify1) {
+      ret = stri_list2matrix(ret, Rf_ScalarLogical(TRUE),
+         stri__vector_NA_strings(1));
+   }
+
    STRI__UNPROTECT_ALL
    return ret;
    STRI__ERROR_HANDLER_END({
