@@ -56,6 +56,10 @@
  *
  * @version 0.2-2 (Marek Gagolewski, 2014-04-20)
  *          new method: countCodePoints()
+ *
+ * @version 0.3-1 (Marek Gagolewski, 2014-11-02)
+ *          BUGFIX?: Added explicit zero bytes at the end of each array;
+ *          new methods: replaceAllAtPos(), setNA()
  */
 class String8  {
 
@@ -100,7 +104,8 @@ class String8  {
             this->m_memalloc = true; // ignore memalloc val
             this->m_n = n-3;
             this->m_str = new char[this->m_n+1];
-            memcpy(this->m_str, str+3, (size_t)this->m_n+1);
+            memcpy(this->m_str, str+3, (size_t)this->m_n);
+            this->m_str[this->m_n] = '\0';
          }
          else {
             this->m_memalloc = memalloc;
@@ -108,10 +113,12 @@ class String8  {
             if (memalloc) {
                this->m_str = new char[this->m_n+1];
                // memcpy may be very fast in some libc implementations
-               memcpy(this->m_str, str, (size_t)this->m_n+1);
+               memcpy(this->m_str, str, (size_t)this->m_n);
+               this->m_str[this->m_n] = '\0';
             }
             else {
                this->m_str = (char*)(str); // we know what we're doing
+               // str is zero-terminated
             }
          }
       }
@@ -138,6 +145,17 @@ class String8  {
          this->m_str = NULL;
       }
 
+
+      /** destructor */
+      void setNA()
+      {
+         if (this->m_str && this->m_memalloc) {
+            delete [] this->m_str;
+         }
+         this->m_str = NULL;
+      }
+
+
       /** copy constructor */
       String8(const String8& s)
       {
@@ -145,7 +163,8 @@ class String8  {
          this->m_n = s.m_n;
          if (s.m_memalloc) {
             this->m_str = new char[this->m_n+1];
-            memcpy(this->m_str, s.m_str, (size_t)this->m_n+1);
+            memcpy(this->m_str, s.m_str, (size_t)this->m_n);
+            this->m_str[this->m_n] = '\0';
          }
          else {
             this->m_str = s.m_str;
@@ -162,7 +181,8 @@ class String8  {
          this->m_n = s.m_n;
          if (s.m_memalloc) {
             this->m_str = new char[this->m_n+1];
-            memcpy(this->m_str, s.m_str, (size_t)this->m_n+1);
+            memcpy(this->m_str, s.m_str, (size_t)this->m_n);
+            this->m_str[this->m_n] = '\0';
          }
          else {
             this->m_str = s.m_str;
@@ -224,6 +244,66 @@ class String8  {
 
          return i;
       }
+
+
+      /** Replace substrings with a given replacement string
+       *
+       *
+       * @version 0.3-1 (Marek Gagolewski, 2014-11-02)
+       */
+      void replaceAllAtPos(R_len_t buf_size,
+         const char* replacement_cur_s, R_len_t replacement_cur_n,
+         std::deque< std::pair<R_len_t, R_len_t> >& occurrences)
+      {
+#ifndef NDEBUG
+         if (isNA()) throw StriException("String8::isNA() in replaceAllAtPos()");
+#endif
+         char* old_str = this->m_str;
+         int old_n = this->m_n;
+         bool old_memalloc = this->m_memalloc;
+         this->m_str = new char[buf_size+1];
+         this->m_n = buf_size;
+         this->m_memalloc = true;
+
+         R_len_t buf_used = 0;
+         R_len_t jlast = 0;
+
+         std::deque< std::pair<R_len_t, R_len_t> >::iterator iter = occurrences.begin();
+         for (; iter != occurrences.end(); ++iter) {
+            pair<R_len_t, R_len_t> match = *iter;
+            memcpy(m_str+buf_used, old_str+jlast, (size_t)(match.first-jlast));
+            buf_used += match.first-jlast;
+#ifndef NDEBUG
+            if (buf_used > buf_size)
+               throw StriException("!NDEBUG: String8::replaceAllAtPos: buf_used > buf_size");
+#endif
+
+            jlast = match.second;
+            memcpy(m_str+buf_used, replacement_cur_s, (size_t)(replacement_cur_n));
+            buf_used += replacement_cur_n;
+#ifndef NDEBUG
+            if (buf_used > buf_size)
+               throw StriException("!NDEBUG: String8::replaceAllAtPos: buf_used > buf_size");
+#endif
+         }
+
+         memcpy(m_str+buf_used, old_str+jlast, (size_t)(old_n-jlast));
+         buf_used += (old_n-jlast);
+#ifndef NDEBUG
+         if (buf_used > buf_size)
+            throw StriException("!NDEBUG: String8::replaceAllAtPos: buf_used > buf_size");
+#endif
+
+#ifndef NDEBUG
+         if (buf_used != this->m_n)
+            throw StriException("!NDEBUG: String8::replaceAllAtPos: buf_used > buf_size");
+#endif
+         this->m_str[this->m_n] = '\0';
+
+         if (old_str && old_memalloc)
+            delete [] old_str;
+      }
+
 };
 
 #endif
